// ***************************************************************************
// ***************************************************************************
// ***************************************************************************
// ***************************************************************************

#include <stdio.h>
#include "platform.h"
#include "xbasic_types.h"
#include "xstatus.h"
#include "xil_io.h"
#include "xiicps.h"
#include "xuartps.h"
#include "xparameters.h"
#include "xparameters_ps.h"

#define JESD_BASEADDR    XPAR_AXI_JESD204B_RX1_0_BASEADDR
#define DMA_BASEADDR     XPAR_AXI_DMA_0_BASEADDR
#define VDMA_BASEADDR    XPAR_AXI_VDMA_0_BASEADDR
#define DDR_BASEADDR     XPAR_DDR_MEM_BASEADDR
#define UART_BASEADDR    XPS_UART1_BASEADDR
#define SPI_BASEADDR     XPAR_AXI_SPI_0_BASEADDR
#define IIC_BASEADDR     XPS_I2C0_BASEADDR
#define CF_BASEADDR      XPAR_AXI_AD9683_0_BASEADDR
#define CF_CGEN_BASEADDR XPAR_AXI_CLKGEN_0_BASEADDR
#define CF_HDMI_BASEADDR XPAR_AXI_HDMI_TX_36B_0_BASEADDR

#define H_STRIDE(m)      (((m) == 1) ? 1280 : 1920)
#define H_COUNT(m)       (((m) == 1) ? 1650 : 2200)
#define H_ACTIVE(m)      (((m) == 1) ? 1280 : 1920)
#define H_WIDTH(m)       (((m) == 1) ?   40 :   44)
#define H_FP(m)          (((m) == 1) ?  110 :   88)
#define H_BP(m)          (((m) == 1) ?  220 :  148)
#define V_COUNT(m)       (((m) == 1) ?  750 : 1125)
#define V_ACTIVE(m)      (((m) == 1) ?  720 : 1080)
#define V_WIDTH(m)       (((m) == 1) ?    5 :    5)
#define V_FP(m)          (((m) == 1) ?    5 :    4)
#define V_BP(m)          (((m) == 1) ?   20 :   36)

#define H_DE_MIN(m)      (H_WIDTH(m) + H_BP(m))
#define H_DE_MAX(m)      (H_WIDTH(m) + H_BP(m) + H_ACTIVE(m))
#define V_DE_MIN(m)      (V_WIDTH(m) + V_BP(m))
#define V_DE_MAX(m)      (V_WIDTH(m) + V_BP(m) + V_ACTIVE(m))
#define VIDEO_LENGTH(m)  (H_ACTIVE(m) * V_ACTIVE(m))
#define VIDEO_BASEADDR   DDR_BASEADDR + 0x2000000

// ***************************************************************************
// ***************************************************************************

extern char inbyte(void);
void xil_printf(const char *ctrl1, ...);
void Xil_DCacheFlush(void);

void delay_ms(u32 ms_count) {
  u32 count;
  for (count = 0; count < ((ms_count * 800000) + 1); count++) {
    asm("nop");
  }
}

u32 user_exit(void) {
  while (XUartPs_IsReceiveData(UART_BASEADDR)) {
    if (inbyte() == 'q') {
      return(1);
    }
  }
  return(0);
}

void ad9683_spi_write(u32 addr, u32 data) {
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x1e6); // control register (rst fifos)
  Xil_Out32((SPI_BASEADDR + 0x70), 0x002); // chip select
  Xil_Out32((SPI_BASEADDR + 0x68), ((addr>>8) & 0xff)); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), (addr & 0xff)); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), data); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x180); // control register
  delay_ms(1);
}

u32 ad9683_spi_read(u32 addr) {
  u32 rdata;
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x1e6); // control register (rst fifos)
  Xil_Out32((SPI_BASEADDR + 0x70), 0x002); // chip select
  Xil_Out32((SPI_BASEADDR + 0x68), (0x80 | ((addr>>8) & 0xff))); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), (addr & 0xff)); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), 0x000); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  rdata = Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x180); // control register
  delay_ms(1);
  return(rdata);
}

void ad9683_spi_verify(u32 addr, u32 data) {
  u32 rdata;
  rdata = ad9683_spi_read(addr);
  if (rdata != data) {
    xil_printf("AD9683[%03x]: rcv(%02x), exp(%02x) !!!\n\r", addr, rdata, data);
  }
  return;
}

// ***************************************************************************
// ***************************************************************************

void adc_capture(u32 qwcnt, u32 sa) {
  Xil_Out32((DMA_BASEADDR + 0x030), 0); // clear dma operations
  Xil_Out32((DMA_BASEADDR + 0x030), 1); // enable dma operations
  Xil_Out32((DMA_BASEADDR + 0x048), sa); // capture start address
  Xil_Out32((DMA_BASEADDR + 0x058), (qwcnt * 8)); // number of bytes
  Xil_Out32((CF_BASEADDR + 0x00c), 0x0); // capture disable
  Xil_Out32((CF_BASEADDR + 0x010), 0xf); // clear status
  Xil_Out32((CF_BASEADDR + 0x014), 0xf); // clear status
  Xil_Out32((CF_BASEADDR + 0x00c), (0x80000000 | (qwcnt - 1)));
  do {delay_ms(1);}
  while ((Xil_In32(CF_BASEADDR + 0x010) & 0x1) == 1);
  if ((Xil_In32(CF_BASEADDR + 0x010) & 0x02) == 0x02) {
    xil_printf("overflow occured, capture may be corrupted\n\r");
  }
  Xil_DCacheFlush();
}

void adc_test(u32 mode, u32 format) {
  u32 n;
  u32 rdata;
  u32 edata;
  edata = 0;
  ad9683_spi_write(0x14, format);
  ad9683_spi_write(0x0d, mode);
  ad9683_spi_write(0xff, 0x01);
  adc_capture(16, DDR_BASEADDR);
  xil_printf("adc_test: mode(%2d), format(%2d)\n\r", mode, format);
  if ((mode == 0x5) || (mode == 0x6)) {
    if (format == 0x1) return;
    Xil_Out32((CF_BASEADDR+0x24), ((mode == 0x5) ? 0x01 : 0x00));
    delay_ms(10);
    Xil_Out32((CF_BASEADDR+0x14), 0xffff);
    delay_ms(100);
    if ((Xil_In32(CF_BASEADDR+0x14) & 0xffff) != 0) {
      xil_printf("  ERROR: PN status(%04x).\n\r", Xil_In32(CF_BASEADDR+0x14));
    }
    return;
  }
  for (n = 0; n < 64; n++) {
    rdata = Xil_In32(DDR_BASEADDR+((n/2)*4));
    rdata = ((n%2) == 1) ? ((rdata>>16) & 0xffff) : ((rdata>> 0) & 0xffff);
    if ((mode == 0x1) && (format == 0)) edata = 0x2000;
    if ((mode == 0x2) && (format == 0)) edata = 0x3fff;
    if ((mode == 0x3) && (format == 0)) edata = 0x0000;
    if ((mode == 0x1) && (format == 1)) edata = 0x0000;
    if ((mode == 0x2) && (format == 1)) edata = 0x1fff;
    if ((mode == 0x3) && (format == 1)) edata = 0x2000;
    if (((mode == 0x4) || (mode == 0x7)) && (n == 0)) edata = (rdata & 0xffff);
    if ((mode == 0x4) && (n > 0)) edata = (edata == 0x2aaa) ? 0x1555 : 0x2aaa;
    if ((mode == 0x7) && (n > 0)) edata = (edata == 0x3fff) ? 0x0000 : 0x3fff;
    if (rdata != edata) {
      xil_printf("  ERROR[%2d]: rcv(%08x), exp(%08x)\n\r", n, rdata, edata);
    }
  }
}

u32 ad9683_setup() {

  u32 mode;

  xil_printf("AD9683[0x000]: %02x\n\r", ad9683_spi_read(0x000));
  xil_printf("AD9683[0x001]: %02x\n\r", ad9683_spi_read(0x001));
  xil_printf("AD9683[0x002]: %02x\n\r", ad9683_spi_read(0x002));

  Xil_Out32((JESD_BASEADDR + 0x0c), 0x03); // reset
  Xil_Out32((JESD_BASEADDR + 0x04), 0x000c1f01); // lanesync, scramble, frm and byte counts
  delay_ms(100);

  ad9683_spi_write(0x5f, (0x16 | 0x1)); // trail bits, ilas normal & pd
  ad9683_spi_write(0x5e, 0x11); // m=1, l=1
  ad9683_spi_write(0x6e, 0x80); // scr, 1-lane
  ad9683_spi_write(0x70, 0x1f); // no. of frames per multi frame
  ad9683_spi_write(0x3a, 0x1e); // sysref enabled
  ad9683_spi_write(0x5f, (0x16 | 0x0)); // enable
  ad9683_spi_write(0x14, 0x00); // offset binary
  ad9683_spi_write(0x0d, 0x00); // test patterns
  ad9683_spi_write(0xff, 0x01);
  ad9683_spi_write(0xff, 0x00);

  ad9683_spi_verify(0x6e, 0x80); // scr, 1-lane
  ad9683_spi_verify(0x70, 0x1f); // no. of frames per multi frame
  ad9683_spi_verify(0x3a, 0x1e); // sysref ctrl
  ad9683_spi_verify(0x5f, 0x16); // enable
  ad9683_spi_verify(0x14, 0x00); // offset binary
  ad9683_spi_verify(0x0d, 0x00); // test patterns
  ad9683_spi_verify(0x6f, 0x01); // bytes per frame
  delay_ms(100);

  if (ad9683_spi_read(0x0a) != 0x81) {
    xil_printf("AD9683, PLL/link errors!!\n\r");
  }

  if (Xil_In32((JESD_BASEADDR + 0x14)) == 0x0) {
    xil_printf("JESD link/lane errors!!\n\r");
    return(0);
  }

  for (mode = 1; mode <= 7; mode++) {
    adc_test(mode, 0x0);
    adc_test(mode, 0x1);
  }

  ad9683_spi_write(0x14, 0x00);
  ad9683_spi_write(0x0d, 0x00);
  ad9683_spi_write(0xff, 0x01);
  return(0);
}

// ***************************************************************************
// ***************************************************************************

void iic_select(u32 sel) {
  u32 rdata;
  Xil_Out32((IIC_BASEADDR+0x00), 0x3f4e);
  Xil_Out32((IIC_BASEADDR+0x10), 0xff);
  Xil_Out32((IIC_BASEADDR+0x14), 0x01);
  Xil_Out32((IIC_BASEADDR+0x0c), sel);
  Xil_Out32((IIC_BASEADDR+0x08), 0x74);
  while ((Xil_In32(IIC_BASEADDR+0x10) & 0x01) == 0x00) {}
  delay_ms(10);
  Xil_Out32((IIC_BASEADDR+0x00), 0x3f4f);
  Xil_Out32((IIC_BASEADDR+0x10), 0xff);
  Xil_Out32((IIC_BASEADDR+0x14), 0x01);
  Xil_Out32((IIC_BASEADDR+0x0c), 0x00);
  Xil_Out32((IIC_BASEADDR+0x08), 0x74);
  while ((Xil_In32(IIC_BASEADDR+0x04) & 0x20) == 0x00) {}
  rdata = Xil_In32(IIC_BASEADDR+0x0c) & 0xff;
  xil_printf("IIC select(%02x).\n\r", rdata);
  delay_ms(10);
}

void iic_write(u32 daddr, u32 waddr, u32 wdata) {
  Xil_Out32((IIC_BASEADDR+0x00), 0x3f4e);
  Xil_Out32((IIC_BASEADDR+0x10), 0xff);
  Xil_Out32((IIC_BASEADDR+0x14), 0x02);
  Xil_Out32((IIC_BASEADDR+0x0c), waddr);
  Xil_Out32((IIC_BASEADDR+0x0c), wdata);
  Xil_Out32((IIC_BASEADDR+0x08), daddr);
  while ((Xil_In32(IIC_BASEADDR+0x10) & 0x01) == 0x00) {}
  delay_ms(10);
}

u32 iic_read(u32 daddr, u32 raddr, u32 display) {
  u32 rdata;
  Xil_Out32((IIC_BASEADDR+0x00), 0x3f5e);
  Xil_Out32((IIC_BASEADDR+0x10), 0xff);
  Xil_Out32((IIC_BASEADDR+0x14), 0x01);
  Xil_Out32((IIC_BASEADDR+0x0c), raddr);
  Xil_Out32((IIC_BASEADDR+0x08), daddr);
  while ((Xil_In32(IIC_BASEADDR+0x10) & 0x01) == 0x00) {}
  Xil_Out32((IIC_BASEADDR+0x00), 0x3f5f);
  Xil_Out32((IIC_BASEADDR+0x10), 0xff);
  Xil_Out32((IIC_BASEADDR+0x14), 0x01);
  Xil_Out32((IIC_BASEADDR+0x0c), 0x00);
  Xil_Out32((IIC_BASEADDR+0x08), daddr);
  while ((Xil_In32(IIC_BASEADDR+0x04) & 0x20) == 0x00) {}
  Xil_Out32((IIC_BASEADDR+0x00), 0x3f0f);
  rdata = Xil_In32(IIC_BASEADDR+0x0c) & 0xff;
  if (display == 1) {
    xil_printf("IIC read(%04x, %04x. %04x).\n\r", daddr, raddr, rdata);
  }
  delay_ms(10);
  return(rdata);
}

// ***************************************************************************
// ***************************************************************************

void ddr_video_wr(u32 m) {

  u32 n;
  u32 p;
  u32 dcnt;

  dcnt = 0;

  xil_printf("ddr write started\n\r");
  for (n = 0; n < V_ACTIVE(m); n++) {
    for (p = 0; p < H_ACTIVE(m); p++) {
      Xil_Out32((VIDEO_BASEADDR + (n*H_STRIDE(m)*4) + (p*4)), 0xFF);
      dcnt = dcnt + 1;
    }
  }
  xil_printf("ddr write completed (total %d)\n\r", dcnt);
  Xil_DCacheFlush();
}

// ***************************************************************************
// ***************************************************************************

void hdmi_setup(u32 m) {

  u32 data;

  delay_ms(1);
  data = Xil_In32(CF_CGEN_BASEADDR + (0x1f*4));
  if ((data & 0x1) == 0x0) {
    xil_printf("pixel clock (148.5MHz) out of lock (0x%04x)\n\r", data);
    return;
  }

  if (m == 1) {
    Xil_Out32((CF_CGEN_BASEADDR + (0x01*4)), 0x0000);
    Xil_Out32((CF_CGEN_BASEADDR + (0x02*4)), 0x01c7);
    Xil_Out32((CF_CGEN_BASEADDR + (0x03*4)), 0x0000);
    Xil_Out32((CF_CGEN_BASEADDR + (0x04*4)), 0x2083);
    Xil_Out32((CF_CGEN_BASEADDR + (0x05*4)), 0x034d);
    Xil_Out32((CF_CGEN_BASEADDR + (0x06*4)), 0x0000);
    Xil_Out32((CF_CGEN_BASEADDR + (0x07*4)), 0x0177);
    Xil_Out32((CF_CGEN_BASEADDR + (0x08*4)), 0x7c01);
    Xil_Out32((CF_CGEN_BASEADDR + (0x09*4)), 0x7fe9);
    Xil_Out32((CF_CGEN_BASEADDR + (0x0a*4)), 0x9000);
    Xil_Out32((CF_CGEN_BASEADDR + (0x0b*4)), 0x0800);
    Xil_Out32((CF_CGEN_BASEADDR + (0x01*4)), 0x0001);
    delay_ms(1);
    data = Xil_In32(CF_CGEN_BASEADDR + (0x1f*4));
    if ((data & 0x1) == 0x0) {
      xil_printf("pixel clock (74.25MHz) out of lock (0x%04x)\n\r", data);
      return;
    }
  }

  ddr_video_wr(m);
  iic_select(0x2);

  Xil_Out32((VDMA_BASEADDR + 0x000), 0x4); // reset
  Xil_Out32((VDMA_BASEADDR + 0x000), 0x0); // reset
  Xil_Out32((VDMA_BASEADDR + 0x000), 0x3); // enable circular mode
  Xil_Out32((VDMA_BASEADDR + 0x05c), VIDEO_BASEADDR); // start address
  Xil_Out32((VDMA_BASEADDR + 0x060), VIDEO_BASEADDR); // start address
  Xil_Out32((VDMA_BASEADDR + 0x064), VIDEO_BASEADDR); // start address
  Xil_Out32((VDMA_BASEADDR + 0x058), (H_STRIDE(m)*4)); // h offset (2048 * 4) bytes
  Xil_Out32((VDMA_BASEADDR + 0x054), (H_ACTIVE(m)*4)); // h size (1920 * 4) bytes
  Xil_Out32((VDMA_BASEADDR + 0x050), V_ACTIVE(m)); // v size (1080)

  Xil_Out32((CF_HDMI_BASEADDR + 0x04), 0x0); // tpg & csc-bypass & enable
  Xil_Out32((CF_HDMI_BASEADDR + 0x08), ((H_WIDTH(m) << 16) | H_COUNT(m)));
  Xil_Out32((CF_HDMI_BASEADDR + 0x0c), ((H_DE_MIN(m) << 16) | H_DE_MAX(m)));
  Xil_Out32((CF_HDMI_BASEADDR + 0x10), ((V_WIDTH(m) << 16) | V_COUNT(m)));
  Xil_Out32((CF_HDMI_BASEADDR + 0x14), ((V_DE_MIN(m) << 16) | V_DE_MAX(m)));
  Xil_Out32((CF_HDMI_BASEADDR + 0x04), 0x1); // tpg & csc-bypass & enable

  // wait for hpd
  while ((iic_read(0x39, 0x96, 0x01) & 0x80) != 0x80) {
    delay_ms(1);
  }

  iic_write(0x39, 0x01, 0x00);
  iic_write(0x39, 0x02, 0x18);
  iic_write(0x39, 0x03, 0x00);
  iic_write(0x39, 0x15, 0x00);
  iic_write(0x39, 0x16, 0x30);
  iic_write(0x39, 0x18, 0x46);
  iic_write(0x39, 0x19, 0x62);
  iic_write(0x39, 0x40, 0x80);
  iic_write(0x39, 0x41, 0x10);
  iic_write(0x39, 0x48, 0x00);
  iic_write(0x39, 0x49, 0xa8);
  iic_write(0x39, 0x4c, 0x00);
  iic_write(0x39, 0x55, 0x00);
  iic_write(0x39, 0x56, 0x08);
  iic_write(0x39, 0x96, 0x20);
  iic_write(0x39, 0x98, 0x03);
  iic_write(0x39, 0x99, 0x02);
  iic_write(0x39, 0x9a, 0xe0);
  iic_write(0x39, 0x9c, 0x30);
  iic_write(0x39, 0x9d, 0x61);
  iic_write(0x39, 0xa2, 0xa4);
  iic_write(0x39, 0xa3, 0xa4);
  iic_write(0x39, 0xa5, 0x44);
  iic_write(0x39, 0xab, 0x40);
  iic_write(0x39, 0xaf, 0x06);
  iic_write(0x39, 0xba, 0x00);
  iic_write(0x39, 0xd0, 0x3c);
  iic_write(0x39, 0xd1, 0xff);
  iic_write(0x39, 0xde, 0x9c);
  iic_write(0x39, 0xe0, 0xd0);
  iic_write(0x39, 0xe4, 0x60);
  iic_write(0x39, 0xf9, 0x00);
  iic_write(0x39, 0xfa, 0x00);
  iic_write(0x39, 0x17, 0x02);
  iic_write(0x39, 0x0a, 0x10);
  iic_write(0x39, 0x0b, 0x8e);
  iic_write(0x39, 0x0c, 0x00);
  iic_write(0x39, 0x73, 0x01);
  iic_write(0x39, 0x14, 0x02);

  iic_read(0x39, 0x42, 0x01);
  iic_read(0x39, 0xc8, 0x01);
  iic_read(0x39, 0x9e, 0x01);
  iic_read(0x39, 0x96, 0x01);
  iic_read(0x39, 0x3e, 0x01);
  iic_read(0x39, 0x3d, 0x01);
  iic_read(0x39, 0x3c, 0x01);

  xil_printf("hdmi setup complete.\n\r");
  return;
}

u32 cnt2rgb(u32 d) {
  if (d > 0x00ffff) return(0xff0000);
  if (d > 0x0000ff) return(0x00ff00);
  if (d > 0x000000) return(0x0000ff);
  return(0);
}

void eyescan_setup(u32 m, u32 x, u32 y, u32 s) {

  u32 v, h, d1, d2;
  u32 sa0, da1, da2;

  Xil_Out32((JESD_BASEADDR + (0x15*4)), (VIDEO_BASEADDR + (H_ACTIVE(m)*y*4))); // start address
  Xil_Out32((JESD_BASEADDR + (0x16*4)), H_ACTIVE(m)); // hsize
  Xil_Out32((JESD_BASEADDR + (0x17*4)), (((x+0x101) << 16) | x)); // hmax, hmin
  Xil_Out32((JESD_BASEADDR + (0x11*4)), 0x0); // prescale
  Xil_Out32((JESD_BASEADDR + (0x12*4)), 0x017f81); // v-offset
  Xil_Out32((JESD_BASEADDR + (0x13*4)), 0x00400fc0); // h-offset
  Xil_Out32((JESD_BASEADDR + (0x14*4)), 0x0001); // h-offset
  Xil_Out32((JESD_BASEADDR + (0x10*4)), ((s == 1) ? 0x2 : 0x0)); // start
  Xil_Out32((JESD_BASEADDR + (0x10*4)), ((s == 1) ? 0x3 : 0x1)); // start
  while ((Xil_In32(JESD_BASEADDR + (0x18*4)) & 0x1) != 0x0) {delay_ms(1);}

  for (v = 0; v < 255; v++) {
    sa0 = VIDEO_BASEADDR + (H_ACTIVE(m)*y*4) + (H_ACTIVE(m)*v*4) + ((x+0x000)*4);
    da1 = VIDEO_BASEADDR + (H_ACTIVE(m)*y*4) + (H_ACTIVE(m)*v*4) + ((x+0x200)*4);
    da2 = VIDEO_BASEADDR + (H_ACTIVE(m)*y*4) + (H_ACTIVE(m)*v*4) + ((x+0x300)*4);
    for (h = 0; h < 129; h++) {
      d1 = Xil_In32(sa0);
      sa0 = sa0 + 0x4;
      d2 = Xil_In32(sa0);
      sa0 = sa0 + 0x4;
      if (s == 1) {
        Xil_Out32(da1, cnt2rgb(((d1>>0) & 0xffff) + ((d1>>16) & 0xffff)));
        Xil_Out32(da2, cnt2rgb(((d2>>0) & 0xffff) + ((d2>>16) & 0xffff)));
      } else {
        Xil_Out32(da1, cnt2rgb(d1));
        Xil_Out32(da2, cnt2rgb(d2));
      }
      da1 = da1 + 0x4;
      da2 = da2 + 0x4;
    }
  }
}

// ***************************************************************************
// ***************************************************************************

int main() {

  init_platform();
  hdmi_setup(0);
  ad9683_setup();

  xil_printf("running eye scan..\n\r");
  eyescan_setup(0, 0x50, 0x050, 0);
  eyescan_setup(0, 0x50, 0x250, 1);
  xil_printf("done\n\r");
  cleanup_platform();
  return 0;
}


// ***************************************************************************
// ***************************************************************************

